@@ -16,9 +16,9 @@
 ; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 ; USA
 
-(define suits-four #t)
-(define suits-two #f)
 (define suits-one #f)
+(define suits-two #f)
+(define suits-four #t)
 
 ;set up the deck
 (set-ace-low)
@@ -45,6 +45,8 @@
 
 (define winning-score 96)
 
+(define allow-empty-slots #f)
+
 (define (new-game)
   (initialize-playing-area)
   (make-deck)
@@ -120,7 +122,8 @@
   #t)
 
 (define (check-for-points slot)
-  (and (is-visible? (cadr (get-cards slot)))
+  (and (> (length (get-cards slot)) 1)
+       (is-visible? (cadr (get-cards slot)))
        (eq? (get-suit (get-top-card slot))
             (get-suit (cadr (get-cards slot))))
        (= (+ 1 (get-value (get-top-card slot)))
@@ -165,16 +168,11 @@
   (and (droppable? start-slot card-list end-slot)
        (complete-transaction start-slot card-list end-slot)))
 
-(define (any-slot-empty? slots)
-  (if (eq? slots '())
-      #f
-      (or (empty-slot? (car slots))
-          (any-slot-empty? (cdr slots)))))
-
 (define (button-clicked slot)
   (and (= stock slot)
        (not (empty-slot? stock))
-       (if (any-slot-empty? tableau)
+       (if (and (not allow-empty-slots)
+                (any-slot-empty? tableau))
 	   (begin
              (set-statusbar-message (_"Please fill in empty pile first."))
              #f)
@@ -183,8 +181,22 @@
 	     (give-status-message)
 	     #t))))
 
+
+(define (is-playable-stack cards suit n)
+  (and (not (null? cards))
+       (= (get-value (car cards)) n)
+       (is-visible? (car cards))
+       (eq? (get-suit (car cards)) suit)
+       (or (= n 13)
+           (is-playable-stack (cdr cards) suit (+ n 1)))))
+
 (define (button-double-clicked slot)
-  #f)
+  (and (member slot tableau)
+       (is-playable-stack (get-cards slot) (get-suit (get-cards slot)) 1)
+       (let ((card-list (list-head (get-cards slot) 13)))
+            (remove-n-cards slot 13)
+            (complete-transaction slot card-list (find-empty-slot foundation)))
+       #t))
 
 (define (game-over)
   (and (not (game-won))
@@ -270,15 +282,15 @@
 
 (define (get-options)
   (list 'begin-exclusive 
-	(list (_"One Suit") suits-one)
-	(list (_"Two Suits") suits-two)
 	(list (_"Four Suits") suits-four)
+	(list (_"Two Suits") suits-two)
+	(list (_"One Suit") suits-one)
 	'end-exclusive))
 
 (define (apply-options options)
-  (set! suits-one (cadr (list-ref options 1)))
+  (set! suits-four (cadr (list-ref options 1)))
   (set! suits-two (cadr (list-ref options 2)))
-  (set! suits-four (cadr (list-ref options 3))))
+  (set! suits-one (cadr (list-ref options 3))))
 
 (define (timeout) #f)
 
