@@ -110,7 +110,11 @@ class GtkViewArea(gtk.DrawingArea):
         glDrawable = gtk.gtkgl.widget_get_gl_drawable(self)
         glContext = gtk.gtkgl.widget_get_gl_context(self)
 
-        # OpenGL begin.
+        # Check were able to get context
+        if glDrawable is None or glContext is None:
+            return
+
+        # OpenGL begin (can fail)
         if not glDrawable.gl_begin(glContext):
             return
         
@@ -130,10 +134,8 @@ class GtkViewArea(gtk.DrawingArea):
         
     def __endGL(self):
         """Free the OpenGL context"""
-        if not self.renderGL:
+        if self.__glDrawable is None or not self.renderGL:
             return
-        
-        assert(self.__glDrawable is not None)
         self.__glDrawable.gl_end()
         self.__glDrawable = None
         
@@ -155,7 +157,9 @@ class GtkViewArea(gtk.DrawingArea):
         """Gtk+ signal"""
         if self.renderGL:
             self.__startGL()
-        
+            if self.__glDrawable is None:
+                return
+
             # Get the scene rendered
             try:
                 if self.view.feedback is not None:
