@@ -22,11 +22,11 @@ if (sys.modules.has_key('gnome_sudoku') and
     BASE_DIR = os.path.join(APP_DATA_DIR,'gnome-sudoku')
     PUZZLE_DIR = os.path.join(APP_DATA_DIR,'gnome-sudoku','puzzles')    
 else:
-    APP_DATA_DIR = os.path.join('/home/jclinton/gnome-games', 'gnome-sudoku', 'data') 
-    IMAGE_DIR = os.path.join('/home/jclinton/gnome-games', 'gnome-sudoku', 'images')
+    APP_DATA_DIR = os.path.join('/home/thomas/svnbranch/gnome-games', 'gnome-sudoku', 'data') 
+    IMAGE_DIR = os.path.join('/home/thomas/svnbranch/gnome-games', 'gnome-sudoku', 'images')
     LOCALEDIR = os.path.join(APP_DATA_DIR, 'locale')
-    GLADE_DIR = os.path.join('/home/jclinton/gnome-games', 'gnome-sudoku', 'glade')
-    BASE_DIR = os.path.join('/home/jclinton/gnome-games', 'gnome-sudoku', 'data')
+    GLADE_DIR = os.path.join('/home/thomas/svnbranch/gnome-games', 'gnome-sudoku', 'glade')
+    BASE_DIR = os.path.join('/home/thomas/svnbranch/gnome-games', 'gnome-sudoku', 'data')
     PUZZLE_DIR = BASE_DIR
 
 DOMAIN = 'gnome-games'
@@ -37,7 +37,7 @@ import gtk.glade
 gtk.glade.bindtextdomain (DOMAIN, LOCALEDIR)
 gtk.glade.textdomain (DOMAIN)
 
-VERSION = "2.22.1"
+VERSION = "2.22.1.1"
 APPNAME = _("GNOME Sudoku")
 APPNAME_SHORT = _("Sudoku")
 COPYRIGHT = 'Copyright \xc2\xa9 2005-2008, Thomas M. Hinkle'
