@@ -456,7 +456,7 @@ const levelinfo level[] = {
    7, 9, 148,
    "###.###"
    "#a...b#"
-   "#cceff#" "#ddegg#" "#dijjg#" "#hijjk#" "#hh*kk#" "#l***l#" "#######"},
+   "#cceff#" "#ddegg#" "#dijjg#" "#hijjk#" "#hh*kk#" "#l***m#" "#######"},
 
   /* puzzle name */
   {N_("Climb 24 Pro"), 3,
@@ -1630,7 +1630,7 @@ about_cb (GtkAction * action)
 			 "comments", _("Sliding Block Puzzles\n\n"
 			 "Klotski is a part of GNOME Games."),
 			 "copyright",
-			 "Copyright \xc2\xa9 1999-2007 Lars Rydlinge",
+			 "Copyright \xc2\xa9 1999-2008 Lars Rydlinge",
 			 "license", license,
                          "wrap-license", TRUE,
                          "authors", authors,
