@@ -1,5 +1,5 @@
 /* games-files.c
-   Copyright 2003 Callum McKenzie
+   Copyright © 2003 Callum McKenzie
 
    This library is free software; you can redistribute it and'or modify
    it under the terms of the GNU Library General Public License as published 
@@ -22,17 +22,32 @@
  * that a game might be interested in. I'm writing them as I perceive
  * and so some of them should be replaced or removed eventually. */
 
-#include <gtk/gtk.h>
+#include <config.h>
+
+#include <string.h>
 #include <stdarg.h>
+
+#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+#ifdef G_OS_WIN32
+#define STRICT
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <io.h>
+#ifndef S_IWUSR
+# define S_IWUSR _S_IWRITE
+#endif
+#endif
+
 #include "games-files.h"
+#include "games-card.h"
 
 G_DEFINE_TYPE (GamesFileList, games_file_list, G_TYPE_OBJECT)
 
 /* Remove duplicate names form the list */
-static void
-games_file_list_remove_duplicates (GamesFileList * filelist)
+     static void
+       games_file_list_remove_duplicates (GamesFileList * filelist)
 {
   GList *l;
 
@@ -134,8 +149,8 @@ games_file_list_transform_basename (GamesFileList * filelist)
   games_file_list_remove_duplicates (filelist);
 }
 
-GSList *image_suffix_list = NULL;
-GStaticMutex image_suffix_mutex = G_STATIC_MUTEX_INIT;
+static GSList *image_suffix_list = NULL;
+static GStaticMutex image_suffix_mutex = G_STATIC_MUTEX_INIT;
 
 /* We only want to initilise the list of suffixes once, this is
  * the function that does it. It might even be thread safe, not that
@@ -275,7 +290,7 @@ games_file_list_new_images (gchar * path1, ...)
  **/
 GtkWidget *
 games_file_list_create_widget (GamesFileList * gamesfilelist,
-			       gchar * selection, guint flags)
+			       const gchar * selection, guint flags)
 {
   gint itemno;
   GtkComboBox *widget;
@@ -297,7 +312,8 @@ games_file_list_create_widget (GamesFileList * gamesfilelist,
      * seriously mangle unicode strings. */
     if (flags & GAMES_FILE_LIST_REMOVE_EXTENSION) {
       s = g_strrstr (visible, ".");
-      *s = '\0';
+      if (s)
+	*s = '\0';
     }
     if (flags & GAMES_FILE_LIST_REPLACE_UNDERSCORES) {
       s = visible;
@@ -309,11 +325,13 @@ games_file_list_create_widget (GamesFileList * gamesfilelist,
     }
 
     gtk_combo_box_append_text (widget, visible);
-    if (selection && (!g_utf8_collate (string, selection))) {
+    if (selection && (!strcmp (string, selection))) {
       gtk_combo_box_set_active (widget, itemno);
       found = TRUE;
     }
 
+    g_free (visible);
+
     itemno++;
     filelist = g_list_next (filelist);
   }
@@ -384,9 +402,9 @@ games_file_list_get_nth (GamesFileList * filelist, gint n)
 }
 
 static void
-games_file_list_finalize (GObject *object)
+games_file_list_finalize (GObject * object)
 {
-  GamesFileList * filelist = GAMES_FILE_LIST (object);
+  GamesFileList *filelist = GAMES_FILE_LIST (object);
 
   /* For simplicity we haven't used the dispose method since we can
    * guarantee that everything this references doesn't reference itself. */
@@ -408,5 +426,172 @@ games_file_list_class_init (GamesFileListClass * class)
 static void
 games_file_list_init (GamesFileList * filelist)
 {
-  filelist->list = NULL;
+}
+
+/**
+ * games_file_list_card_themes:
+ * @scalable: whether to look for scalable or prerendered themes
+ *
+ * Note that the returned #GamesFileList's list contains the found
+ * theme names in the filename encoding!
+ * 
+ * Returns: a new #GamesFileList containing the found themes
+ */
+GamesFileList *
+games_file_list_card_themes (gboolean scalable)
+{
+  GamesFileList *files;
+  GList *l;
+  const char *glob, *ext, *dir;
+
+#ifdef HAVE_RSVG
+  if (scalable) {
+    glob = "*.svg";
+    ext = ".svg";
+    dir = SCALABLE_CARDS_DIR;
+  } else
+#endif /* HAVE_RSVG */
+  {
+    glob = "*.card-theme";
+    ext = ".card-theme";
+    dir = PRERENDERED_CARDS_DIR;
+  }
+
+  files = games_file_list_new (glob, dir, NULL);
+  games_file_list_transform_basename (files);
+
+  for (l = files->list; l != NULL; l = l->next) {
+    const char *filename = (const char *) l->data;
+    char *dot;
+
+    dot = g_strrstr (filename, ext);
+    if (dot) {
+      *dot = '\0';
+    }
+  }
+
+  return files;
+}
+
+#if defined (G_OS_WIN32) && defined (PREFIX)
+
+static const gchar *
+games_toplevel_directory (void)
+{
+  static gchar *toplevel = NULL;
+
+  if (toplevel)
+    return toplevel;
+
+  {
+    gchar *filename;
+    gchar *sep1, *sep2;
+
+    if (G_WIN32_HAVE_WIDECHAR_API ()) {
+      wchar_t w_filename[MAX_PATH];
+
+      if (GetModuleFileNameW (NULL,
+			      w_filename, G_N_ELEMENTS (w_filename)) == 0)
+	g_error ("GetModuleFilenameW failed");
+
+      filename = g_utf16_to_utf8 (w_filename, -1, NULL, NULL, NULL);
+      if (filename == NULL)
+	g_error ("Converting module filename to UTF-8 failed");
+    } else {
+      gchar cp_filename[MAX_PATH];
+
+      if (GetModuleFileNameA (NULL,
+			      cp_filename, G_N_ELEMENTS (cp_filename)) == 0)
+	g_error ("GetModuleFilenameA failed");
+
+      filename = g_locale_to_utf8 (cp_filename, -1, NULL, NULL, NULL);
+      if (filename == NULL)
+	g_error ("Converting module filename to UTF-8 failed");
+    }
+
+    /* If the executable file name is of the format
+     * <foobar>\bin\*.exe , use <foobar>.
+     * Otherwise, use the directory where the executable is.
+     */
+
+    sep1 = strrchr (filename, '\\');
+    *sep1 = '\0';
+
+    sep2 = strrchr (filename, '\\');
+    if (sep2 != NULL) {
+      *sep2 = '\0';
+    }
+
+    toplevel = filename;
+  }
+
+  return toplevel;
+}
+#endif
+
+/**
+ * games_path_runtime_fix_private:
+ * @path: A pointer to a string (allocated with g_malloc) that is
+ *        (or could be) a pathname.
+ *
+ * On Windows, this function checks if the string pointed to by @path
+ * starts with the compile-time prefix, and in that case, replaces the
+ * prefix with the run-time one.  @path should be a pointer to a
+ * dynamically allocated (with g_malloc, g_strconcat, etc) string. If
+ * the replacement takes place, the original string is deallocated,
+ * and *@path is replaced with a pointer to a new string with the
+ * run-time prefix spliced in.
+ *
+ * On Linux, it does the same thing, but only if BinReloc support is enabled.
+ * On other Unices, it does nothing because those platforms don't have a
+ * way to find out where our binary is.
+ */
+static void
+games_path_runtime_fix_private (gchar **path)
+{
+#if defined (G_OS_WIN32) && defined (PREFIX)
+  gchar *p;
+
+      /* This is a compile-time entry. Replace the path with the
+       * real one on this machine.
+       */
+      p = *path;
+      *path = g_strconcat (games_toplevel_directory (),
+                           "\\",
+                           *path + strlen (PREFIX "/"),
+                           NULL);
+      g_free (p);
+  /* Replace forward slashes with backslashes, just for
+   * completeness */
+  p = *path;
+  while ((p = strchr (p, '/')) != NULL)
+    {
+      *p = '\\';
+      p++;
+    }
+#endif
+}
+
+
+gchar *
+games_path_runtime_fix (const gchar *path)
+{
+  gchar *p = g_strdup (path);
+  games_path_runtime_fix_private (&p);
+
+  return p;
+}
+
+
+gchar *
+games_build_filename (const gchar * path, const gchar * filename)
+{
+  gchar *p = g_strdup (path);
+	  
+  games_path_runtime_fix_private (&p);
+
+  gchar *result = g_build_filename (p, filename, NULL);
+  g_free (p);
+
+  return result;
 }
